// This file is generated by omniidl (C++ backend)- omniORB_4_2. Do not edit.
#ifndef __boxes_operators_hh__
#define __boxes_operators_hh__

void operator<<=(::CORBA::Any& _a, CORBA::StringValue* _s);
void operator<<=(::CORBA::Any& _a, CORBA::StringValue** _s);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::StringValue*& _s);

void operator<<=(::CORBA::Any& _a, CORBA::WStringValue* _s);
void operator<<=(::CORBA::Any& _a, CORBA::WStringValue** _s);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::WStringValue*& _s);

#endif

