// This file is generated by omniidl (C++ backend)- omniORB_4_2. Do not edit.
#ifndef __TimeBase_hh__
#define __TimeBase_hh__

#ifndef __CORBA_H_EXTERNAL_GUARD__
#include <omniORB4/CORBA.h>
#endif

#ifndef  USE_stub_in_nt_dll
# define USE_stub_in_nt_dll_NOT_DEFINED_TimeBase
#endif
#ifndef  USE_core_stub_in_nt_dll
# define USE_core_stub_in_nt_dll_NOT_DEFINED_TimeBase
#endif
#ifndef  USE_dyn_stub_in_nt_dll
# define USE_dyn_stub_in_nt_dll_NOT_DEFINED_TimeBase
#endif


#include "COS_sysdep.h"



#ifdef USE_stub_in_nt_dll
# ifndef USE_core_stub_in_nt_dll
#  define USE_core_stub_in_nt_dll
# endif
# ifndef USE_dyn_stub_in_nt_dll
#  define USE_dyn_stub_in_nt_dll
# endif
#endif

#ifdef _core_attr
# error "A local CPP macro _core_attr has already been defined."
#else
# ifdef  USE_core_stub_in_nt_dll
#  define _core_attr _OMNIORB_NTDLL_IMPORT
# else
#  define _core_attr
# endif
#endif

#ifdef _dyn_attr
# error "A local CPP macro _dyn_attr has already been defined."
#else
# ifdef  USE_dyn_stub_in_nt_dll
#  define _dyn_attr _OMNIORB_NTDLL_IMPORT
# else
#  define _dyn_attr
# endif
#endif



_CORBA_MODULE TimeBase

_CORBA_MODULE_BEG

  _CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_TimeT;

  typedef ::CORBA::ULongLong TimeT;
  typedef ::CORBA::ULongLong_out TimeT_out;

  _CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_InaccuracyT;

  typedef TimeT InaccuracyT;
  typedef TimeT_out InaccuracyT_out;

  _CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_TdfT;

  typedef ::CORBA::Short TdfT;
  typedef ::CORBA::Short_out TdfT_out;

  struct UtcT {
    typedef _CORBA_ConstrType_Fix_Var<UtcT> _var_type;

    
    TimeT time;

    ::CORBA::ULong inacclo;

    ::CORBA::UShort inacchi;

    TdfT tdf;

  

    void operator>>= (cdrStream &) const;
    void operator<<= (cdrStream &);
  };

  typedef UtcT::_var_type UtcT_var;

  typedef UtcT& UtcT_out;

  _CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_UtcT;

  struct IntervalT {
    typedef _CORBA_ConstrType_Fix_Var<IntervalT> _var_type;

    
    TimeT lower_bound;

    TimeT upper_bound;

  

    void operator>>= (cdrStream &) const;
    void operator<<= (cdrStream &);
  };

  typedef IntervalT::_var_type IntervalT_var;

  typedef IntervalT& IntervalT_out;

  _CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_IntervalT;

_CORBA_MODULE_END



_CORBA_MODULE POA_TimeBase
_CORBA_MODULE_BEG

_CORBA_MODULE_END



_CORBA_MODULE OBV_TimeBase
_CORBA_MODULE_BEG

_CORBA_MODULE_END





#undef _core_attr
#undef _dyn_attr

extern void operator<<=(::CORBA::Any& _a, const TimeBase::UtcT& _s);
extern void operator<<=(::CORBA::Any& _a, TimeBase::UtcT* _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, TimeBase::UtcT*& _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const TimeBase::UtcT*& _sp);

extern void operator<<=(::CORBA::Any& _a, const TimeBase::IntervalT& _s);
extern void operator<<=(::CORBA::Any& _a, TimeBase::IntervalT* _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, TimeBase::IntervalT*& _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const TimeBase::IntervalT*& _sp);





#ifdef   USE_stub_in_nt_dll_NOT_DEFINED_TimeBase
# undef  USE_stub_in_nt_dll
# undef  USE_stub_in_nt_dll_NOT_DEFINED_TimeBase
#endif
#ifdef   USE_core_stub_in_nt_dll_NOT_DEFINED_TimeBase
# undef  USE_core_stub_in_nt_dll
# undef  USE_core_stub_in_nt_dll_NOT_DEFINED_TimeBase
#endif
#ifdef   USE_dyn_stub_in_nt_dll_NOT_DEFINED_TimeBase
# undef  USE_dyn_stub_in_nt_dll
# undef  USE_dyn_stub_in_nt_dll_NOT_DEFINED_TimeBase
#endif

#endif  // __TimeBase_hh__

