#pragma once
#ifndef HAVE_ALLOCA_H
/* #undef HAVE_ALLOCA_H */
#endif

#ifndef HAVE_ALLOCA
/* #undef HAVE_ALLOCA */
#endif

#ifndef HAVE_ACCESS
/* #undef HAVE_ACCESS */
#endif

#ifndef HAVE_BOOL
#define HAVE_BOOL 1
#endif

#ifndef HAVE_CATCH_BY_BASE
#define HAVE_CATCH_BY_BASE 1
#endif

#ifndef HAVE_CONST_CAST
#define HAVE_CONST_CAST 1
#endif

#ifndef HAVE_DYNAMIC_CAST
#define HAVE_DYNAMIC_CAST 1
#endif

#ifndef HAVE_REINTERPRET_CAST
#define HAVE_REINTERPRET_CAST 1
#endif

#ifndef HAVE_MEMBER_CONSTANTS
#define HAVE_MEMBER_CONSTANTS 1
#endif

#ifndef HAVE_NAMESPACES
#define HAVE_NAMESPACES 1
#endif

#ifndef HAVE_EXCEPTIONS
#define HAVE_EXCEPTIONS 1
#endif

#ifndef HAVE_STD
#define HAVE_STD 1
#endif

#ifndef HAVE_STDIO_H
#define HAVE_STDIO_H 1
#endif

#ifndef HAVE_SIG_IGN
#define HAVE_SIG_IGN 1
#endif

#ifndef HAVE_STRUCT_LIFCONF
/* #undef HAVE_STRUCT_LIFCONF */
#endif

#ifndef HAVE_STRUCT_LIFCONF
/* #undef HAVE_STRUCT_LIFCONF */
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN6
#define HAVE_STRUCT_SOCKADDR_IN6 1
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
/* #undef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN */
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN_SIN_ZERO
#define HAVE_STRUCT_SOCKADDR_IN_SIN_ZERO 1
#endif

#ifndef HAVE_STRUCT_SOCKADDR_STORAGE
#define HAVE_STRUCT_SOCKADDR_STORAGE 1
#endif

#ifndef OMNI_SOCKNAME_SIZE_T
#define OMNI_SOCKNAME_SIZE_T socklen_t
#endif

/* define if you want to use CFNetwork */
#ifndef OMNI_USE_CFNETWORK_CONNECT
/* #undef OMNI_USE_CFNETWORK_CONNECT */
#endif

#ifndef HAVE_ERRNO_H
#define HAVE_ERRNO_H 1
#endif

#ifndef HAVE_FCNTL_H
#define HAVE_FCNTL_H 1
#endif

#ifndef HAVE_GETADDRINFO
#define HAVE_GETADDRINFO 1
#endif

#ifndef HAVE_GETHOSTNAME
#define HAVE_GETHOSTNAME 1
#endif

#ifndef HAVE_GETNAMEINFO
#define HAVE_GETNAMEINFO 1
#endif

#ifndef HAVE_GETOPT
/* #undef HAVE_GETOPT */
#endif

#ifndef HAVE_GETTIMEOFDAY
/* #undef HAVE_GETTIMEOFDAY */
#endif

#ifndef GETTIMEOFDAY_TIMEZONE
/* #undef GETTIMEOFDAY_TIMEZONE */
#endif

#ifndef HAVE_IFADDRS_H
/* #undef HAVE_IFADDRS_H */
#endif

#ifndef HAVE_INET_NTOP
/* #undef HAVE_INET_NTOP */
#endif

#ifndef HAVE_INSINFF
/* #undef HAVE_INSINFF */
#endif

#ifndef HAVE_INTTYPES_H
#define HAVE_INTTYPES_H 1
#endif

#ifndef HAVE_ISINF
/* #undef HAVE_ISINF */
#endif

#ifndef HAVE_ISINFL
/* #undef HAVE_ISINFL */
#endif

#ifndef HAVE_ISNANORINF
/* #undef HAVE_ISNANORINF */
#endif

#ifndef HAVE_LOCALTIME
/* #undef HAVE_LOCALTIME */
#endif

#ifndef HAVE_MEMORY_H
#define HAVE_MEMORY_H 1
#endif

#ifndef HAVE_NANOSLEEP
/* #undef HAVE_NANOSLEEP */
#endif

#ifndef HAVE_NAN_H
/* #undef HAVE_NAN_H */
#endif

#ifndef HAVE_NETDB_H
/* #undef HAVE_NETDB_H */
#endif

#ifndef HAVE_POLL
/* #undef HAVE_POLL */
#endif

#ifndef HAVE_RAND_R
/* #undef HAVE_RAND_R */
#endif

#ifndef HAVE_SIGACTION
/* #undef HAVE_SIGACTION */
#endif

#ifndef HAVE_SIGNAL_H
#define HAVE_SIGNAL_H 1
#endif

#ifndef HAVE_SIGVEC
/* #undef HAVE_SIGVEC */
#endif

#ifndef HAVE_SNPRINTF
#define HAVE_SNPRINTF 1
#endif

#ifndef HAVE_STDINT_H
#define HAVE_STDINT_H 1
#endif

#ifndef HAVE_STDLIB_H
#define HAVE_STDLIB_H 1
#endif

#ifndef HAVE_STRCASECMP
/* #undef HAVE_STRCASECMP */
#endif

#ifndef HAVE_STRDUP
#define HAVE_STRDUP 1
#endif

#ifndef HAVE_STRERROR
#define HAVE_STRERROR 1
#endif

#ifndef HAVE_STRFTIME
/* #undef HAVE_STRFTIME */
#endif

#ifndef HAVE_STRICMP
#define HAVE_STRICMP 1
#endif

#ifndef HAVE_STRINGS_H
/* #undef HAVE_STRINGS_H */
#endif

#ifndef HAVE_STRING_H
#define HAVE_STRING_H 1
#endif

#ifndef HAVE_STRNCASECMP
/* #undef HAVE_STRNCASECMP */
#endif

#ifndef HAVE_STRTOUL
#define HAVE_STRTOUL 1
#endif

#ifndef HAVE_STRTOULL
#define HAVE_STRTOULL 1
#endif

#ifndef HAVE_STRTOUQ
/* #undef HAVE_STRTOUQ */
#endif

#ifndef HAVE_SYS_IF_H
/* #undef HAVE_SYS_IF_H */
#endif

#ifndef HAVE_SYS_IOCTL_H
/* #undef HAVE_SYS_IOCTL_H */
#endif

#ifndef HAVE_SYS_PARAM_H
/* #undef HAVE_SYS_PARAM_H */
#endif

#ifndef HAVE_SYS_POLL_H
/* #undef HAVE_SYS_POLL_H */
#endif

#ifndef HAVE_SYS_STAT_H
#define HAVE_SYS_STAT_H 1
#endif

#ifndef HAVE_SYS_TIME_H
/* #undef HAVE_SYS_TIME_H */
#endif

#ifndef HAVE_SYS_TYPES_H
#define HAVE_SYS_TYPES_H 1
#endif

#ifndef HAVE_SYS_TYPES_H
#define HAVE_SYS_TYPES_H 1
#endif

#ifndef HAVE_UNAME
/* #undef HAVE_UNAME */
#endif

#ifndef HAVE_UNISTD_H
/* #undef HAVE_UNISTD_H */
#endif

#ifndef HAVE_VPRINTF
#define HAVE_VPRINTF 1
#endif

#ifndef HAVE_VSNPRINTF
#define HAVE_VSNPRINTF 1
#endif

#ifndef STACK_DIRECTION
#define STACK_DIRECTION -1
#endif


#define SIZEOF_WCHAR_T 2

#define SIZEOF_BOOL 1
#define SIZEOF_CHAR 1
#define SIZEOF_DOUBLE 8
#define SIZEOF_FLOAT 4
#define SIZEOF_INT 4
#define SIZEOF_LONG 4
#define SIZEOF_LONG_DOUBLE 8
#define SIZEOF_LONG_LONG 8
#define SIZEOF_SHORT 2
#define SIZEOF_UNSIGNED_CHAR 1
#define SIZEOF_VOIDP 8


#ifndef STDC_HEADERS
#define STDC_HEADERS 1
#endif

/* #undef WORDS_BIGENDIAN */

#ifndef OMNI_LONG_IS_INT
#define OMNI_LONG_IS_INT 1
#endif



#ifndef OMNIORB_DISABLE_ALLOCA
/* #undef OMNIORB_DISABLE_ALLOCA */
#endif

#ifndef OMNIORB_DISABLE_LONGDOUBLE
/* #undef OMNIORB_DISABLE_LONGDOUBLE */
#endif

#ifndef OMNIORB_ENABLE_LOCK_TRACES
/* #undef OMNIORB_ENABLE_LOCK_TRACES */
#endif

#ifndef OMNI_DISABLE_ATOMIC_OPS
/* #undef OMNI_DISABLE_ATOMIC_OPS */
#endif

#ifndef OMNI_DISABLE_IPV6
/* #undef OMNI_DISABLE_IPV6 */
#endif

