// This file is generated by omniidl (C++ backend)- omniORB_4_2. Do not edit.
#ifndef __corbaidl_operators_hh__
#define __corbaidl_operators_hh__

extern void operator<<=(::CORBA::Any& _a, const CORBA::StructMember& _s);
extern void operator<<=(::CORBA::Any& _a, CORBA::StructMember* _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::StructMember*& _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::StructMember*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::StructMemberSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::StructMemberSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::StructMemberSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::StructMemberSeq*& _sp);

extern void operator<<=(::CORBA::Any& _a, const CORBA::UnionMember& _s);
extern void operator<<=(::CORBA::Any& _a, CORBA::UnionMember* _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::UnionMember*& _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::UnionMember*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::UnionMemberSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::UnionMemberSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::UnionMemberSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::UnionMemberSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::EnumMemberSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::EnumMemberSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::EnumMemberSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::EnumMemberSeq*& _sp);

extern void operator<<=(::CORBA::Any& _a, const CORBA::ValueMember& _s);
extern void operator<<=(::CORBA::Any& _a, CORBA::ValueMember* _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::ValueMember*& _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::ValueMember*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::ValueMemberSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::ValueMemberSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::ValueMemberSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::ValueMemberSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::AnySeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::AnySeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::AnySeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::AnySeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::BooleanSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::BooleanSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::BooleanSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::BooleanSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::CharSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::CharSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::CharSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::CharSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::WCharSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::WCharSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::WCharSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::WCharSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::OctetSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::OctetSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::OctetSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::OctetSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::ShortSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::ShortSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::ShortSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::ShortSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::UShortSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::UShortSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::UShortSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::UShortSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::LongSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::LongSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::LongSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::LongSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::ULongSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::ULongSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::ULongSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::ULongSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::LongLongSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::LongLongSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::LongLongSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::LongLongSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::ULongLongSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::ULongLongSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::ULongLongSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::ULongLongSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::FloatSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::FloatSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::FloatSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::FloatSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::DoubleSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::DoubleSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::DoubleSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::DoubleSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::StringSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::StringSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::StringSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::StringSeq*& _sp);

void operator<<=(::CORBA::Any& _a, const CORBA::WStringSeq& _s);
void operator<<=(::CORBA::Any& _a, CORBA::WStringSeq* _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, CORBA::WStringSeq*& _sp);
_CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const CORBA::WStringSeq*& _sp);

#endif

