// -*- Mode: C++; -*-
//
// Edit this file to include any local customisation
//

#ifndef __LOCAL_CONFIG_H__
#define __LOCAL_CONFIG_H__

// If you do not want to map CORBA::WChar to C++ wchar_t and instead want
// it to map to a 16-bit entity, uncomment the following two lines:

// #define _CORBA_WCHAR_DECL unsigned short
// #define SIZEOF_WCHAR      2


// If you want operators for wide strings in wide streams, uncomment
// the following:

// #define OMNI_WIDE_STREAM_OPERATORS

#endif
