#pragma once
#ifndef HAVE_ALLOCA_H
#define HAVE_ALLOCA_H
#endif

#ifndef HAVE_ALLOCA
/* #undef HAVE_ALLOCA */
#endif

#ifndef HAVE_ACCESS
#define HAVE_ACCESS 1
#endif

#ifndef HAVE_STRUCT_LIFCONF
/* #undef HAVE_STRUCT_LIFCONF */
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN6
/* #undef HAVE_STRUCT_SOCKADDR_IN6 */
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
/* #undef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN */
#endif

#ifndef HAVE_STRUCT_SOCKADDR_IN_SIN_ZERO
/* #undef HAVE_STRUCT_SOCKADDR_IN_SIN_ZERO */
#endif

#ifndef HAVE_STRUCT_SOCKADDR_STORAGE
/* #undef HAVE_STRUCT_SOCKADDR_STORAGE */
#endif

#ifndef HAVE_ERRNO_H
#define HAVE_ERRNO_H 1
#endif

#ifndef HAVE_FCNTL_H
#define HAVE_FCNTL_H 1
#endif

#ifndef HAVE_GETADDRINFO
#define HAVE_GETADDRINFO 1
#endif

#ifndef HAVE_GETHOSTNAME
#define HAVE_GETHOSTNAME 1
#endif

#ifndef HAVE_GETNAMEINFO
#define HAVE_GETNAMEINFO 1
#endif

#ifndef HAVE_GETOPT
#define HAVE_GETOPT 1
#endif

#ifndef HAVE_GETTIMEOFDAY
#define HAVE_GETTIMEOFDAY 1
#endif

#ifndef HAVE_IFADDRS_H
#define HAVE_IFADDRS_H 1
#endif

#ifndef HAVE_INET_NTOP
#define HAVE_INET_NTOP 1
#endif

#ifndef HAVE_INSINFF
/* #undef HAVE_INSINFF */
#endif

#ifndef HAVE_INTTYPES_H
#define HAVE_INTTYPES_H 1
#endif

#ifndef HAVE_ISINF
#define HAVE_ISINF 1
#endif

#ifndef HAVE_ISINFL
#define HAVE_ISINFL 1
#endif

#ifndef HAVE_ISNANORINF
/* #undef HAVE_ISNANORINF */
#endif

#ifndef HAVE_LOCALTIME
#define HAVE_LOCALTIME 1
#endif

#ifndef HAVE_MEMORY_H
#define HAVE_MEMORY_H 1
#endif

#ifndef HAVE_NANOSLEEP
#define HAVE_NANOSLEEP 1
#endif

#ifndef HAVE_NAN_H
/* #undef HAVE_NAN_H */
#endif

#ifndef HAVE_NETDB_H
#define HAVE_NETDB_H 1
#endif

#ifndef HAVE_POLL
#define HAVE_POLL 1
#endif

#ifndef HAVE_RAND_R
#define HAVE_RAND_R 1
#endif

#ifndef HAVE_SIGACTION
#define HAVE_SIGACTION 1
#endif

#ifndef HAVE_SIGNAL_H
#define HAVE_SIGNAL_H 1
#endif

#ifndef HAVE_SIGVEC
/* #undef HAVE_SIGVEC */
#endif

#ifndef HAVE_SNPRINTF
#define HAVE_SNPRINTF 1
#endif

#ifndef HAVE_STDINT_H
#define HAVE_STDINT_H 1
#endif

#ifndef HAVE_STDLIB_H
#define HAVE_STDLIB_H 1
#endif

#ifndef HAVE_STRCASECMP
#define HAVE_STRCASECMP 1
#endif

#ifndef HAVE_STRDUP
#define HAVE_STRDUP 1
#endif

#ifndef HAVE_STRERROR
#define HAVE_STRERROR 1
#endif

#ifndef HAVE_STRFTIME
#define HAVE_STRFTIME 1
#endif

#ifndef HAVE_STRICMP
/* #undef HAVE_STRICMP */
#endif

#ifndef HAVE_STRINGS_H
#define HAVE_STRINGS_H 1
#endif

#ifndef HAVE_STRING_H
#define HAVE_STRING_H 1
#endif

#ifndef HAVE_STRNCASECMP
#define HAVE_STRNCASECMP 1
#endif

#ifndef HAVE_STRTOUL
#define HAVE_STRTOUL 1
#endif

#ifndef HAVE_STRTOULL
#define HAVE_STRTOULL 1
#endif

#ifndef HAVE_STRTOUQ
#define HAVE_STRTOUQ 1
#endif

#ifndef HAVE_SYS_IF_H
/* #undef HAVE_SYS_IF_H */
#endif

#ifndef HAVE_SYS_IOCTL_H
#define HAVE_SYS_IOCTL_H 1
#endif

#ifndef HAVE_SYS_PARAM_H
#define HAVE_SYS_PARAM_H 1
#endif

#ifndef HAVE_SYS_POLL_H
#define HAVE_SYS_POLL_H 1
#endif

#ifndef HAVE_SYS_STAT_H
#define HAVE_SYS_STAT_H 1
#endif

#ifndef HAVE_SYS_TIME_H
#define HAVE_SYS_TIME_H 1
#endif

#ifndef HAVE_SYS_TYPES_H
#define HAVE_SYS_TYPES_H 1
#endif

#ifndef HAVE_SYS_TYPES_H
#define HAVE_SYS_TYPES_H 1
#endif

#ifndef HAVE_UNAME
#define HAVE_UNAME 1
#endif

#ifndef HAVE_UNISTD_H
#define HAVE_UNISTD_H 1
#endif

#ifndef HAVE_VPRINTF
#define HAVE_VPRINTF 1
#endif

#ifndef HAVE_VSNPRINTF
#define HAVE_VSNPRINTF 1
#endif


#define SIZEOF_WCHAR_T 4
/* #undef SIZEOF_BOOL */
#define SIZEOF_CHAR 1
#define SIZEOF_DOUBLE 8
#define SIZEOF_FLOAT 4
#define SIZEOF_INT 4
#define SIZEOF_LONG 8
#define SIZEOF_LONG_DOUBLE 16
#define SIZEOF_LONG_LONG 8
#define SIZEOF_SHORT 2
#define SIZEOF_UNSIGNED_CHAR 1
#define SIZEOF_VOIDP 8


#ifndef STDC_HEADERS
#define STDC_HEADERS
#endif

/* #undef WORDS_BIGENDIAN */
