// This file is generated by omniidl (C++ backend)- omniORB_4_2. Do not edit.
#ifndef __poa_enums_defs_hh__
#define __poa_enums_defs_hh__

enum ThreadPolicyValue { ORB_CTRL_MODEL, SINGLE_THREAD_MODEL, MAIN_THREAD_MODEL /*, __max_ThreadPolicyValue=0xffffffff */ };
typedef ThreadPolicyValue& ThreadPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_ThreadPolicyValue;

enum LifespanPolicyValue { TRANSIENT, PERSISTENT /*, __max_LifespanPolicyValue=0xffffffff */ };
typedef LifespanPolicyValue& LifespanPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_LifespanPolicyValue;

enum IdUniquenessPolicyValue { UNIQUE_ID, MULTIPLE_ID /*, __max_IdUniquenessPolicyValue=0xffffffff */ };
typedef IdUniquenessPolicyValue& IdUniquenessPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_IdUniquenessPolicyValue;

enum IdAssignmentPolicyValue { USER_ID, SYSTEM_ID /*, __max_IdAssignmentPolicyValue=0xffffffff */ };
typedef IdAssignmentPolicyValue& IdAssignmentPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_IdAssignmentPolicyValue;

enum ImplicitActivationPolicyValue { IMPLICIT_ACTIVATION, NO_IMPLICIT_ACTIVATION /*, __max_ImplicitActivationPolicyValue=0xffffffff */ };
typedef ImplicitActivationPolicyValue& ImplicitActivationPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_ImplicitActivationPolicyValue;

enum ServantRetentionPolicyValue { RETAIN, NON_RETAIN /*, __max_ServantRetentionPolicyValue=0xffffffff */ };
typedef ServantRetentionPolicyValue& ServantRetentionPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_ServantRetentionPolicyValue;

enum RequestProcessingPolicyValue { USE_ACTIVE_OBJECT_MAP_ONLY, USE_DEFAULT_SERVANT, USE_SERVANT_MANAGER /*, __max_RequestProcessingPolicyValue=0xffffffff */ };
typedef RequestProcessingPolicyValue& RequestProcessingPolicyValue_out;

_CORBA_MODULE_VAR _dyn_attr const ::CORBA::TypeCode_ptr _tc_RequestProcessingPolicyValue;

#endif

