// This file is generated by omniidl (C++ backend)- omniORB_4_2. Do not edit.
#ifndef __ziop_operators_hh__
#define __ziop_operators_hh__

extern void operator<<=(::CORBA::Any& _a, const ZIOP::CompressedData& _s);
extern void operator<<=(::CORBA::Any& _a, ZIOP::CompressedData* _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, ZIOP::CompressedData*& _sp);
extern _CORBA_Boolean operator>>=(const ::CORBA::Any& _a, const ZIOP::CompressedData*& _sp);

#endif

